unit Main;

// TODO: Also include BC++ Versions
{$IFNDEF BCB}
{$DEFINE DEL1UP}
{$IFNDEF VER80}
{$DEFINE DEL2UP}
{$IFNDEF VER90}
{$DEFINE DEL3UP}
{$IFNDEF VER100}
{$DEFINE DEL4UP}
{$IFNDEF VER120}
{$DEFINE DEL5UP}
{$IFNDEF VER130}
{$DEFINE DEL6UP}
{...}
{$ENDIF}
{$ENDIF}
{$ENDIF}
{$ENDIF}
{$ENDIF}
{$ENDIF}

{$IFDEF DEL6UP}
{$WARN UNIT_PLATFORM OFF}
{$ENDIF}

interface

uses
  Windows, SysUtils, Forms, Controls, StdCtrls,
  ExtCtrls, Classes, Dialogs;

type
  TMainForm = class(TForm)
    dlg_open: TOpenDialog;
    mmo_output: TMemo;
    pnl_top: TPanel;
    pnl_right: TPanel;
    btn_readown: TButton;
    pnl_left: TPanel;
    btn_read: TButton;
    Panel1: TPanel;
    dlg_save: TSaveDialog;
    Button2: TButton;
    Button1: TButton;
    procedure btn_read_click(Sender: TObject);
    procedure btn_readown_click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  public
    procedure ReadRecycler(filename: string);
  end;

var
  MainForm: TMainForm;

implementation

uses
  RecyclerFunctions;

{$R *.dfm}

function _BoolToEnabledDisabled(b: boolean): string;
begin
  if b then
    result := 'Enabled'
  else
    result := 'Disabled';
end;

function _BoolToYesNo(b: boolean): string;
begin
  if b then
    result := 'Yes'
  else
    result := 'No';
end;

{$IFNDEF DEL6UP}
// Only available since Delphi 6
function DirectoryExists(const directory: string): boolean;
var
  attr: integer;
begin
  attr := getfileattributes(pchar(directory));
  result := (attr <> -1) and (file_attribute_directory and attr <> 0);
end;
{$ENDIF}

procedure TMainForm.ReadRecycler(filename: string);
var
  x: TStringList;
  i: integer;
  tmp, fn: string;
begin
  mmo_output.Visible := false;
  application.ProcessMessages;

  if (uppercase(copy(extractfilename(filename), 0, 2)) = '$I') or
     (uppercase(copy(extractfilename(filename), 0, 2)) = '$R') then
  begin
    mmo_output.Lines.Add('Reading directory:');
    tmp := extractfilepath(filename);
  end
  else
  begin
    tmp := filename;
    if directoryexists(tmp) then
      mmo_output.Lines.Add('Reading directory:')
    else
      mmo_output.Lines.Add('Reading file:');
  end;

  mmo_output.Lines.Add(tmp);
  mmo_output.lines.add('');

  if not RecyclerIsValid(filename) then
  begin
    mmo_output.lines.add('Recycler is not valid.');
    mmo_output.lines.add('');
  end
  else
  begin
    x := TStringList.Create;
    try
      RecyclerListIndexes(filename, x);
      if x.Count = 0 then
      begin
        mmo_output.lines.add('No items available.');
        mmo_output.lines.add('');
      end;
      for i := 0 to x.Count - 1 do
      begin
        fn := RecyclerCurrentFilenameAndPath(tmp, x[i]);
        mmo_output.lines.add(inttostr(i+1)+'. Entry');
        mmo_output.lines.add('# Unique ID: ' + changefileext(x[i], ''));
        mmo_output.lines.add('# Recycler filename: '+fn);

        if fileexists(fn) then
          mmo_output.lines.add('  -> Found')
        else
          mmo_output.lines.add('  -> Not found');

        {$IFDEF DEL6UP}
        mmo_output.lines.add('# Deleted: '+
          datetimetostr(RecyclerGetDateTime(tmp, x[i])));
        {$ELSE}
        mmo_output.lines.add('# Deleted: You have to compile this demo with Delphi 6 or higher.');
        {$ENDIF}
        mmo_output.lines.add('# Original filename: '+
          RecyclerGetSource(tmp, x[i]));
        mmo_output.lines.add('# Unicode filename: '+
          RecyclerGetSourceUnicode(tmp, x[i]));
        mmo_output.lines.add('# Source device: '+
          RecyclerGetSourceDrive(tmp, x[i]));
        mmo_output.lines.add('# Original size on disk: '+
          inttostr(RecyclerOriginalSize(tmp, x[i]))+' Byte');
        mmo_output.lines.add('');
      end;
    finally
      x.Free;
    end;
  end;

  mmo_output.Visible := true;
end;

procedure TMainForm.btn_read_click(Sender: TObject);
begin
  if dlg_open.Execute() then
  begin
    mmo_output.Clear;
    ReadRecycler(dlg_open.filename);
  end;
end;

procedure TMainForm.btn_readown_click(Sender: TObject);
var
  x, sl: TStringList;
  i, j: integer;
  somethingfound: boolean;
begin
  mmo_output.Visible := false;
  try
    application.ProcessMessages;

    mmo_output.Clear;
    x := TStringList.Create;
    try
      RecyclerGetAllRecyclerDrives(x);
      somethingfound := false;
      if x.Count > 0 then
      begin
        for i := 0 to x.Count - 1 do
        begin
          // ReadRecycler(RecyclerGetPath(x[i][1]), true);

          sl := TStringList.Create;
          try
            RecyclerGetInfofiles(x[i][1], true, sl);
            if sl.Count > 0 then
            begin
              for j := 0 to sl.Count - 1 do
              begin
                ReadRecycler(sl.Strings[j]);
                somethingfound := true;
              end;
            end;
          finally
            sl.Free;
          end;
        end;
      end;
      if not somethingfound then
      begin
        mmo_output.lines.add('No recyclers found.');
        mmo_output.lines.add('');
      end;
    finally
      x.Free;
    end;
  finally
    mmo_output.Visible := true;
  end;
end;

procedure TMainForm.Button1Click(Sender: TObject);
begin
  if dlg_save.Execute then
  begin
    mmo_output.Lines.SaveToFile(dlg_save.FileName);
  end;
end;

procedure TMainForm.Button2Click(Sender: TObject);
var
  d: char;
  sl: TStringList;
  i: integer;
begin
  mmo_output.Visible := false;
  try
    application.ProcessMessages;

    mmo_output.Clear;

    mmo_output.Lines.Add(RecyclerLibraryVersion);

    mmo_output.Lines.Add('');
    mmo_output.Lines.Add('= Current status =');
    mmo_output.Lines.Add('');

    if RecyclerQueryFunctionAvailable then
    begin
      mmo_output.Lines.Add('GLOBAL Empty = ' + _BoolToYesNo(RecyclerIsEmpty));
      mmo_output.Lines.Add('GLOBAL Number of items = ' + IntToStr(RecyclerGetNumItems));
      mmo_output.Lines.Add('GLOBAL Size = ' + IntToStr(RecyclerGetSize) + ' Bytes');
      for d := 'A' to 'Z' do
      begin
        mmo_output.Lines.Add('Drive '+d+' Empty = ' + _BoolToYesNo(RecyclerIsEmpty(d)));
        mmo_output.Lines.Add('Drive '+d+' Number of items = ' + IntToStr(RecyclerGetNumItems(d)));
        mmo_output.Lines.Add('Drive '+d+' Size = ' + IntToStr(RecyclerGetSize(d)) + ' Bytes');
      end;
    end
    else
    begin
      mmo_output.Lines.Add('Empty, Number of items, Size:');
      mmo_output.Lines.Add('        Functionality not working with your operating system.');
    end;

    mmo_output.Lines.Add('');
    mmo_output.Lines.Add('= Name and Infotips =');
    mmo_output.Lines.Add('');

    mmo_output.Lines.Add('Name: '+RecyclerGetName());
    mmo_output.Lines.Add('Info Tip: '+RecyclerGetInfoTip());
    mmo_output.Lines.Add('Intro Text: '+RecyclerGetIntroText());
    mmo_output.Lines.Add('Class-ID: '+RecyclerGetCLSID());

    mmo_output.Lines.Add('');
    mmo_output.Lines.Add('= Icons =');
    mmo_output.Lines.Add('');

    mmo_output.Lines.Add('Default Icon: '+RecyclerGetDefaultIconString());
    if RecyclerQueryFunctionAvailable then
    begin
      mmo_output.Lines.Add('Current Icon (Empty / Full): '+RecyclerGetCurrentIconString());
    end
    else
    begin
      mmo_output.Lines.Add('Current Icon (Empty / Full):');
      mmo_output.Lines.Add('        Functionality not working with your operating system.');
    end;
    mmo_output.Lines.Add('Full Icon: '+RecyclerGetFullIconString());
    mmo_output.Lines.Add('Empty Icon: '+RecyclerGetEmptyIconString());

    mmo_output.Lines.Add('');
    mmo_output.Lines.Add('= Events =');
    mmo_output.Lines.Add('');

    mmo_output.Lines.Add('Empty Event Name: '+RecyclerEmptyEventGetName());
    mmo_output.Lines.Add('Empty Event Current Sound: '+RecyclerEmptyEventGetCurrentSound());
    mmo_output.Lines.Add('Empty Event Default Sound: '+RecyclerEmptyEventGetDefaultSound());

    sl := TStringList.Create;
    try
      RecyclerEmptyEventGetSoundCategories(sl);
      for i := 0 to sl.Count - 1 do
      begin
        mmo_output.Lines.Add(Format('Event "%s" = %s', [sl.Strings[i], RecyclerEmptyEventGetSound(sl.Strings[i])]));
      end;
    finally
      sl.Free;
    end;

    mmo_output.Lines.Add('');
    mmo_output.Lines.Add('= Nuke on Delete =');
    mmo_output.Lines.Add('');

    mmo_output.Lines.Add('Global settings selected: ' + _BoolToEnabledDisabled(RecyclerHasGlobalSettings));
    mmo_output.Lines.Add('Global setting: ' + _BoolToEnabledDisabled(RecyclerGlobalIsNukeOnDelete()));
    for d := 'A' to 'Z' do
    begin
      mmo_output.Lines.Add('Setting for drive '+d+': ' + _BoolToEnabledDisabled(RecyclerSpecificIsNukeOnDelete(d)));
    end;

    mmo_output.Lines.Add('');
    mmo_output.Lines.Add('= Usage Percent =');
    mmo_output.Lines.Add('');

    mmo_output.Lines.Add('Global settings selected: ' + _BoolToEnabledDisabled(RecyclerHasGlobalSettings));
    mmo_output.Lines.Add('Global setting: ' + IntToStr(RecyclerGlobalGetPercentUsage()));
    for d := 'A' to 'Z' do
    begin
      mmo_output.Lines.Add('Setting for drive '+d+': ' + IntToStr(RecyclerSpecificGetPercentUsage(d)));
    end;

    mmo_output.Lines.Add('');
    mmo_output.Lines.Add('= Confirmation Dialog =');
    mmo_output.Lines.Add('');

    mmo_output.Lines.Add('Setting: ' + _BoolToEnabledDisabled(RecyclerConfirmationDialogEnabled()));
  finally
    mmo_output.Visible := true;
  end;
end;

end.
